

		The C4510R0 Microcontroller Chip

      The C4510R0 is a one-chip microcontroller minus RAM minus ROM. It
contains the following hardware...

	1 4502	microprocessor core
	1	memory mapper (1 Mbyte)
	2 6526	peripheral cores
	1 6518	style UART
	1	power-on reset logic

C4510R0 pinouts...

	Quan  Title	Function

	  8   DB0-DB7	Bidirectional Data Bus
	 20   A0-A19	Address Buss		(tristated by AEC low)
	  1   NOMAP	No mapped memory	(output)
	  1   NOIO	Chip select internal I/O (input)
	  1   R/W	Read/Write		(tristated by AEC low)
	  1   CPUCLK	CPU Clock		(input)
	  1   AEC	Address Enable		(input)
	  1   /NMI	non-maskable interrupt	(input, pulled-up)
	  1   /RESTORE  timed NMI		(input, pulled-up)
	  1   /IRQ	interrupt request	(input, pulled-up)
	  1   /RESET	reset			(output, big, pulled-up)
	  1   /DMA	external DMA request	(input from expansion)

	 10   Col0-Col9	Keyboard Columns
	  8   Row0-Row7	Keyboard Rows
	  1   /SRQIN
	  1   CNT1
	  1   SP1

	  1   SRQATN
	  1   SRQCLK	Serial buss stuff
	  1   SRQDATA
	  1   PA2
	  8   PB0-PB7
	  1   /PC2
	  1   /FLAG2
	  1   CNT2
	  1   SP2

	  1   TXD	uart			(output)
	  1   RXD	uart			(input)

	  1   DOTCLK	7 mhz clock		(input)
	  1   TEST	Test			(input)

	  2   VCC
	  1   GND


	Summary of C4510R0 micro-core enhancements over the c4502R1 core

      The microprocessor core is actually a C4502R1 with some additional
instructions, used to operate the memory mapper.

      The former AUG (augment) opcode has been changed to MAP (mapper),
and the former NOP (no-operation) has been changed to EOM (end-of-mapping-
sequence).

		How the Memory Mapper Works

      The C4510R0 memory mapper allows the microprocessor to access up to 1
megabyte of memory. Here's how. The 6502 microprocessor can only access 64K
bytes of memory because it only uses addresses of 16 bits. The 4502 is no
different, nor is the 4510. But the 4510 memory mapper allows these addresses
to be redirected to new physical addresses to access different parts of a
much larger memory, within the 64K byte confinement window.

      The 64K window has been divided into eight blocks, and two regions,
with four blocks in each region. Blocks 0 through 3 are in the "lower"
region, and blocks 4 through 7 are in the "upper" region, as shown...
	

(0		l	lqqqqqqqqqqqkFFFF
		x	x   BLK 7   x
		x	tqqqqqqqqqqquE000
		x	x   BLK 6   x
   UPPER REGION u	tqqqqqqqqqqquC000
		x	x   BLK 5   x
		x	tqqqqqqqqqqquA000
		x	x   BLK 4   x
		t	tqqqqqqqqqqqu8000
		x	x   BLK 3   x
		x	tqqqqqqqqqqqu6000
		x	x   BLK 2   x
   LOWER REGION u	tqqqqqqqqqqqu4000
		x	x   BLK 1   x
		x	tqqqqqqqqqqqu2000
		x	x   BLK 0   x
		m	mqqqqqqqqqqqj0(B

      Each block can be programmed to be "mapped", or "non-mapped" via
bits in the mapper's "mask" registers. Non-mapped means, simply, address
out equals address in. Therefore, there are still only 64K bytes of non-
mapped memory. Mapped, means that address out equals address in plus some
offset. The offset is programmed via the mapper's "offset" registers.
There are two "offset" registers. One is for the lower region, and one is
for the upper region.

      The low-order 8 addresses are never mapped. The offsets are only
added to the 12 high-order addresses. This means the smallest unit you can
map to is 256 bytes or one page.

      The C4510R0 has an output (NOMAP) which lets the outside world know
when the processor is accessing mapped (0) or non-mapped (1) address. This
is useful for systems where you may want I/O devices to be at fixed (non-
mapped) addresses, and only memory at mapped addresses.

      It is possible, and likely, to have mapped, and unmapped memory at
the save physical address. And, with offset registers set to zero, mapped
addresses will match unmapped ones. The only difference is the NOMAP signal
to tell whether the address is mapped or unmapped.

      To program the mapper, the operating system must load the A, X, Y,
and Z reisters with the following information, and execute a MAP opcode.

		Mapper Register Data
(0
    7       6       5       4       3       2       1       0    BIT
lqqqqqqqwqqqqqqqwqqqqqqqwqqqqqqqwqqqqqqqwqqqqqqqwqqqqqqqwqqqqqqqk
x	x	x	x	x	x	x	x	x
x LOWER	x LOWER	x LOWER	x LOWER	x LOWER	x LOWER	x LOWER	x LOWER	x A
x OFF15	x OFF14	x OFF13	x OFF12	x OFF11	x OFF10	x OFF9	x OFF8	x
x	x	x	x	x	x	x	x	x
tqqqqqqqnqqqqqqqnqqqqqqqnqqqqqqqnqqqqqqqnqqqqqqqnqqqqqqqnqqqqqqqu
x	x	x	x	x	x	x	x	x
x MAP	x MAP	x MAP	x MAP	x LOWER	x LOWER	x LOWER	x LOWER	x X
x BLK3	x BLK2	x BLK1	x BLK0	x OFF19	x OFF18	x OFF17	x OFF16	x
x	x	x	x	x	x	x	x	x
tqqqqqqqnqqqqqqqnqqqqqqqnqqqqqqqnqqqqqqqnqqqqqqqnqqqqqqqnqqqqqqqu
x	x	x	x	x	x	x	x	x
x UPPER	x UPPER	x UPPER	x UPPER	x UPPER	x UPPER	x UPPER	x UPPER	x Y
x OFF15	x OFF14	x OFF13	x OFF12	x OFF11	x OFF10	x OFF9	x OFF8	x
x	x	x	x	x	x	x	x	x
tqqqqqqqnqqqqqqqnqqqqqqqnqqqqqqqnqqqqqqqnqqqqqqqnqqqqqqqnqqqqqqqu
x	x	x	x	x	x	x	x	x
x MAP	x MAP	x MAP	x MAP	x UPPER	x UPPER	x UPPER	x UPPER	x Z
x BLK7	x BLK6	x BLK5	x BLK4	x OFF19	x OFF18	x OFF17	x OFF16	x
x	x	x	x	x	x	x	x	x
mqqqqqqqvqqqqqqqvqqqqqqqvqqqqqqqvqqqqqqqvqqqqqqqvqqqqqqqvqqqqqqqj(B

      After executing the MAP opcode, all interrupts are inhibited. This
is done to allow the operating system to complete a mapping sequence
without fear of getting an interrupt. An interrupt occuring before the
proper stack-pointer is set will cause return address data to be written
to an undesired area.

      Upon completing the mapping sequence, the operating system must remove
the interrupt inhibit by executing a EOM (formerly NOP) opcode. Note that
application software may execute NOPs with no effect.

      Except for the removal of the SO (set-overflow) pin, the remainder of
the C4510R0 is identical to the C4502R1.


	Summary of C4502R1 enhancements over the 65C02

      The C4502R1, upon reset, looks and acts like any other CMOS 6502
processor, with the exception that many instructions are shorter or
require less cycles than they used to. This causes programs to execute
in less time that older versions, even at the same clock frequency.

      The stack pointer has been expanded to 16 bits, but can be used
in two different modes. It can be used as a full 16-bit (word) stack
pointer, or as an 8-bit (byte) pointer whose stack page is programmable.
On reset, the byte mode is selected with page 1 set as the stack page.
This is done to make it fully 65C02 compatible.

      The zero page is also programmable via a new register, the "B" or
"Base Page" register. On reset, this register is cleared, thus giving a
true "zero" page for compatability reasons, but the user can define any
page in memory as the "zero" page.

      A third index register, "Z", has been added to increase flexability
in data manipulation. This register is also cleared, on reset, so that the
STZ instructions still do what they used to, for compatability.

      This is a list of opcodes that have been added to the 210 previously
defined MOS, Rockwell, and GTE opcodes.

1. Branches and Jumps

	BPL label	word-relative
	BMI label	word-relative
	BRU label	word-relative (BRA)
	BVC label	word-relative
	BVS label	word-relative
	BCC label	word-relative
	BCS label	word-relative
	BNE label	word-relative
	BEQ label	word-relative

	BSR label	Branch to subroutine (word relative)
	JSR (ABS)	Jump to subroutine absolute indirect
	JSR (ABS,X)	Jump to subroutine absolute indirect, X
	RTN #		Return from subroutine and adjust stack pointer.

2. Arithmetic Operations

	NEG A		Negate (or 2's complement) accumulator.

	ASR A		Arithmetic Shift right accumulator or memory
	ASR ZP
	ASR ZP,X

	INW ZP		Increment Word
	DEW ZP		Decrement Word

	INZ		Increment and
	DEZ		Decrement Z register

	ASW ABS		Arithmetic Shift Left Word
	ROW ABS		Rotate Left Word

	ORA (ZP),Z	these were formerly (ZP) non-indexed
	AND (ZP),Z	now are indexed by Z register
	EOR (ZP),Z
	ADC (ZP),Z
	CMP (ZP),Z
	SBC (ZP),Z

	CPZ IMM		Compare Z register with memory immediate,
	CPZ ZP		zero page, and
	CPZ ABS		absolute.

3. Loads, Stores, Pushes, Pulls and Transfers

	LDA (ZP),Z	formerly (ZP)

	LDZ IMM		Load Z register immediate,
	LDZ ABS		absolute,
	LDZ ABS,X	absolute,X.

	LDA (d,SP),Y	Load Accum via stack vector indexed by Y
	STA (d,SP),Y	and Store

	STX ABS,Y	Store X Absolute, Y
	STY ABS,X	Store Y Absolute, X

	STZ ZP		Store Z register (formerly store zero)
	STZ ABS
	STZ ZP,X
	STZ ABS,X

	STA (ZP),Z	formerly (ZP)

	PHD IMM		Push Data Immediate (word)
	PHD ABS		Push Data Absolute (word)

	PHZ		Push Z register onto stack
	PLZ		Pull Z register from stack

	TAZ		Transfer Accumulator to Z register
	TZA		Transfer Z register to Accumulator

	TAB		Transfer Accumulator to Base page register
	TBA		Transfer Base page register to Accumulator

	TSY		Transfer Stack Pointer High byte to Y register
			 and set "byte" stack-pointer mode
	TYS		Transfer Y register to Stack Pointer High byte
			 and set "word" stack-pointer mode





			MOS4502R1 Registers

The MOS4502R1 has the following 8 user registers.

	A	accumulator
	X	index-X
	Y	index-Y
	Z	index-Z

	B	Base-page
	P	Processor status
	SP	Stack pointer
	PC	Program counter

Accumulator

      The accumulator is the only general purpose computational register. It
can be used for arithmetic functions add, subtract, shift, rotate, negate,
and for Boolean functions and, or, exclusive-or, and bit operations. It
cannot, however, be used as an index register.

Index X

      The index register X has the largest number of opcodes pertaining to,
or using it. It can be incremented, decremented, or compared, but not used
for arithmetic or logical (Boolean) operations. It differs from other index
registers in that it is the only register that can be used in indexed-indirect
or (bp,X) operations. It cannot be used in indirect-indexed or (bp),Y mode.

Index Y

      The index register Y has the same computational constraints as the X
register, but finds itself in a lot less of the opcodes, making it less
generally used. But the index Y has one advantage over index X, in that it
can be used in indirect-indexed operations or (bp),Y mode.

Index Z

      The index register Z is the most unique, in that it is used in the
smallest number of opcodes. It also has the same computation limitations as
the X and Y registers, but has an extra feature. Upon reset, the Z register
is cleared so that the STZ (store zero) opcodes and non-indexed indirect
opcodes from previous 65C02 designs are emulated. The Z register can also
be used in indirect-indexed or (bp),Z operations.

Base page B register

      Early versions of 6502 microprocessors had a special subset of
instructions that required less code and less time to execute. These were
referred to as the "zero page" instructions. Since the addressing page
was always known, and known to be zero, addresses could be specified as
a single byte, instead of two bytes.

      The MOS4502 also implements this same "zero page" set of instructions,
but goes one step further by allowing the programmer to specify which page
is to be the "zero page". Now that the programmer can program this page, it
is now, not necessarily page zero, but instead, the "selected page". The term
"base page" is used, however.

      The B register selects which page will be the "base page", and the user
sets it by transferring the contents of the accumulator to it. At reset, the
B register is cleared, giving initially a true "zero page".


Processor status P register

      The processor status register is an 8-bit register which is used to
indicate the status of the microprocessor. It contains 8 processor "flags".
Some of the flags are set or reset based on the results of various types
of operations. Others are more specific. The flags are...

      Flag	Name		Typical indication
	N	Negative	result of operation is negative
	V	Overflow	result of add or subtract causes signed overflow
	E	Extend		disables stack pointer extension
	B	Break		interrupt was caused by BRK opcode
	D	Decimal		perform add/subtract using BCD math
	I	Interrupt	disable IRQ interrupts
	Z	Zero		result of operation is zero
	C	Carry		operation caused a carry


Stack Pointer SP

      The stack pointer is a 16 bit register that has two modes. It can be
programmed to be either an 8-bit page programmable pointer, or a full 16-bit
pointer. The processor status E bit selects which mode will be used. When
set, the E bit selects the 8-bit mode. When reset, the E bit selects the
16-bit mode.

      Upon reset, the MOS4502R1 will come up in the 8-bit page-programmable
mode, with the stack page set to 1. This makes it compatible with earlier
6502 products. The programmer can quickly change the default stack page by
loading the Y register with the desired page and transferring its contents 
to the stack pointer high byte, using the TYS opcode. The 8-bit stack pointer
can be set by loading the X register with the desired value, and transferring
its contents to the stack pointer low byte, using the TXS opcode.

      To select the 16-bit stack pointer mode, the user must execute a CLE
(for CLear Extend disable) opcode. Setting the 16-bit pointer is done by
loading the X and Y registers with the desired stack pointer low and high
bytes, respectively, and then transferring their contents to the stack
pointer using TXS and TYS. To return to 8-bit page mode, simple execute a
SEE (SEt Extend disable) opcode.

	************************************************************
	*			WARNING				   *
	*							   *
	*  If you are using Non-Maskable-Interrupts, or Interrupt  *
	*  Request is enabled, and you want to change BOTH stack   *
	*  pointer bytes, do not put any code between the TXS and  *
	*  TYS opcodes. Taking this precaution will prevent any    *
	*  interrupts from occuring between the setting of the two *
	*  stack pointer bytes, causing a potential for writing    *
	*  stack data to an unwanted area.			   *
	************************************************************

Program Counter PC

      The program counter is a 16-bit up-only counter that determines what
area of memory that program information will be fetched from. The user
generally only modifies it using jumps, branches, subroutine calls, or
returns. It is set initially, and by interrupts, from vectors at memory
addresses FFFA through FFFF (hex). See "Interrupts" below.


			MOS4502R1 Interrupts

      There are four basic interrupt sources on the MOS4502R1. These are
RES*, IRQ*, NMI*, and SO, for Reset, Interrupt Request, Non-Maskable
Interrupt, and Set Overflow. The Reset is a hard non-recoverable interrupt
that stops everything. The IRQ is a "maskable" interrupt, in that its
occurance can be prevented. The NMI is "non-maskable", and if such an
event occurs, cannot be prevented. The SO, or Set Overflow, is not really
an interrupt, but causes an externally generated condition, which can be
used for control of program flow.

      One important design feature, which must be remembered is that no
interrupt can occur immediately after a one-cycle opcode. This is very
important, because there are times when you want to temporarily prevent
interrupts from occurring. The best example of this is, when setting a
16-bit stack pointer, you do not want an interrupt to occur between the
times you set the low-order byte, and the high-order byte. If it could
happen, the interrupt would do stack writes using a pointer that was only
partially set, thus, writing to an unwanted area.

IRQ*

      The IRQ* (Interrupt ReQuest) input will cause an interrupt, if it is
at a low logic level, and the I processor status flag is reset. The interrupt
sequence will begin with the first SYNC after a multiple-cycle opcode. The two
program counter bytes PCH and PCL, and the processor status register P, are
pushed onto the stack. (This causes the stack pointer SP to be decremented
by 3.) Then the program counter bytes PCL and PCH are loaded from memory
addresses FFFE and FFFF, respectively.

      An interrupt caused by the IRQ* input, is similar to the BRK opcode,
but differs, as follows. The program counter value stored on the stack points
to the opcode that would have been executed, had the interrupt not occurred.
On return from interrupt, the processor will return to that opcode. Also, when
the P register is pushed onto the stack, the B or "break" flag pushed, is zero, 
to indicate that the interrupt was not software generated.

NMI*

      The NMI* (Non-Maskable Interrupt) input will cause an interrupt after
receiving a high to low transition The interrupt sequence will begin with the
first SYNC after a multiple-cycle opcode. NMI* inputs cannot be masked by
the processor status register I flag. The two program counter bytes PCH and
PCL, and the processor status register P, are pushed onto the stack. (This
causes the stack pointer SP to be decremented by 3.) Then the program counter
bytes PCL and PCH are loaded from memory addresses FFFA and FFFB.

      As with IRQ*, when the P register is pushed onto the stack, the B or
"break" flag pushed, is zero, to indicate that the interrupt was not software
generated.

RES*

      The RES* (reset) input will cause a hard reset instantly as it is
brought to a low logic level. This effects the following conditions. The
currently executing opcode will be terminated. The B and Z registers will
be cleared. The stack pointer will be set to "byte" mode, with the stack
page set to page 1. The processor status bits E and I will be set.

      The RES* input should be held low for at least 2 clock cycles. But
once brought high, the reset sequence begins on the CPU cycle. The first
four cycles of the reset sequence do nothing. Then the program counter
bytes PCL and PCH are loaded from memory addresses FFFC and FFFD, and
normal program execution begins.

SO

      The SO (set overflow) input does, as its name implies, set the
overflow or V processor status flag. The effect is immediate as this active
low signal is brought or held at a low logic level. Care should be taken
if this signal is used, as some of the opcodes can set or reset the
overflow flag, as well.


			MOS4502R0 Addressing Modes

      It should be noted that all 8-bit addresses are referred to as "byte"
addresses, and all 16-bit addresses are referred to as "word" addresses. In
all word addresses, the low-order byte of the address is fetched from the lower
of two consecutive memory addresses, and the high-order byte of the address
is fetched the higher of the two. So, in all operations, the low-order address
is fetched first.


Implied		OPR

      The register or flag affected is identified entirely by the opcode
in this (usually) single cycle instruction. In this document, any implied
operation, where the implied register is not explicitly declared, implies
the accumulator. Example: INC with no arguments implies "increment the
accumulator".

Immediate (byte, word)		OPR #xx

      The data used in the operation is taken from the byte or bytes
immediately following the opcode in the 2-byte or 3-byte instruction.

Base Page			OPR bp		(formerly Zero Page)

      The second byte of the two-byte instruction contains the low-order
address byte, and the B register contains the high-order address byte of
the memory location to be used by the operation.


Base Page, indexed by X		OPR bp,X	(formerly Zero Page,X)

      The second byte of the two-byte instruction is added to the X index
register to form the low-order address byte, and the B register contains the
high-order address byte of the memory location to be used by the operation.

Base Page, indexed by Y		OPR bp,Y	(formerly Zero Page,Y)

      The second byte of the two-byte instruction is added to the Y index
register to form the low-order address byte, and the B register contains the
high-order address byte of the memory location to be used by the operation.

Absolute			OPR abs

      The second and third bytes of the three-byte instruction contain the
low-order and high-order address bytes, respectively, of the memory location
to be used by the operation.

Absolute, indexed by X		OPR abs,X

      The second and third bytes of the three-byte instruction are added to
the unsigned contents of the X index register to form the low-order and
high-order address bytes, respectively, of the memory location to be used
by the operation.

Absolute, indexed by Y		OPR abs,Y

      The second and third bytes of the three-byte instruction are added to
the unsigned contents of the Y index register to form the low-order and
high-order address bytes, respectively, of the memory location to be used
by the operation.

Indirect (word)			OPR (abs)	(JMP and JSR only)

      The second and third bytes of the three-byte instruction contain the
low-order and high-order address bytes, respectively, of two memory locations
containing the low-order and high-order JMP or JSR addresses, respectively.

Indexed by X, indirect (byte)	OPR (bp,X)	(formerly (zp,X) )

      The second byte of the two-byte instruction is added to the contents
of the X register to form the low-order address byte, and the contents of the B
register contains the high-order address byte, of two memory locations that
contain the low-order and high-order address of the memory location to be used
by the operation.

Indexed by X, indirect (word)	OPR (abs,X)	(JMP and JSR only)

      The second and third bytes of the three-byte instruction are added to
the unsigned contents of the X index register to form the low-order and
high-order address bytes, respectively, of two memory locations containing the
low-order and high-order JMP or JSR address bytes.

Indirect, indexed by Y		OPR (bp),Y	(formerly (zp),Y )

      The second byte of the two-byte instruction contains the low-order byte,
and the B register contains the high-order address byte of two memory loc-
ations whose contents are added to the unsigned Y index register to form the
address of the memory location to be used by the operation.

Indirect, indexed by Z		OPR (bp),Z	(formerly (zp) )

      The second byte of the two-byte instruction contains the low-order byte,
and the B register contains the high-order address byte of two memory loc-
ations whose contents are added to the unsigned Z index register to form the
address of the memory location to be used by the operation.

Stack Pointer Indirect, indexed by Y	OPR (d,SP),Y	(new)

      The second byte of the two-byte instruction contains an unsigned offset
value, d, which is added to the stack pointer (word) to form the address of
two memory locations whose contents are added to the unsigned Y register to
form the address of the memory location to be used by the operation.

Relative (byte)			Bxx LABEL	(branches only)

      The second byte of the two-byte branch instruction is sign-extended to
a full word and added to the program counter (now containing the opcode address
plus two). If the condition of the branch is true, the sum is stored back into
the program counter.

Relative (word)			Bxx LABEL	(branches only)

      The socond and third bytes of the three-byte branch instruction are
added to the low-order and high-order program counter bytes, respectively.
(the program counter now contains the opcode address plus two). If the
condition of the branch is true, the sum is stored back into the program
counter.

			MOS4502R1 Instruction Set

Add memory to accumulator with carry                                     ADC

A=A+M+C

  Addressing Mode               Abbrev.    Opcode

  immediate                     IMM         69
  base page                     BP          65
  base page indexed X           BP,X        75
  absolute                      ABS         6D
  absolute indexed X            ABS,X       7D
  absolute indexed Y            ABS,Y       79
  base page indexed indirect X  (BP,X)      61
  base page indirect indexed Y  (BP),Y      71
  base page indirect indexed Z  (BP),Z      72

   Bytes   Cycles     Mode
     2       2      immediate
     2       3      base page non-indexed, or indexed X or Y
     3       4      absolute  non-indexed, or indexed X or Y
     2       5      base page indexed indirect X, or indirect indexed Y or Z

      The ADC instructions add data fetched from memory and carry to the
contents of the accumulator. The results of the add are then stored in the
accumulator. If the "D" or Decimal Mode flag, in the processor status regis-
ter, then a Binary Coded Decimal (BCD) add is performed.

      The "N" or Negative flag will be set if the sum is negative, otherwise
it is cleared. The "V" or Overflow flag will be set if the sign of the sum
is different from the sign of both addends, indicating a signed overflow.
Otherwise, it is cleared. The "Z" or Zero flag is set if the sum (stored into
the accumulator) is zero, otherwise, it is cleared. The "C" or carry is set
if the sum of the unsigned addends exceeds 255 (binary mode) or 99 (decimal
mode).

       Flags
  N V E B D I Z C
  N V - - - - Z C


And memory logically with accumulator                                    AND

A=A.and.M

  Addressing Mode               Abbrev.    Opcode

  immediate                     IMM         29
  base page                     BP          25
  base page indexed X           BP,X        35
  absolute                      ABS         2D
  absolute indexed X            ABS,X       3D
  absolute indexed Y            ABS,Y       39
  base page indexed indirect X  (BP,X)      21
  base page indirect indexed Y  (BP),Y      31
  base page indirect indexed Z  (BP),Z      32

   Bytes   Cycles     Mode
     2       2      immediate
     2       3      base page non-indexed, or indexed X or Y
     3       4      absolute  non-indexed, or indexed X or Y
     2       5      base page indexed indirect X, or indirect indexed Y or Z

      The AND instructions perform a logical "and" between data bits fetched
from memory and the accumulator bits. The results are then stored in the
accumulator. For each accumulator and corresponding memory bit that are both
logical 1's, the result is a 1. Otherwise it is 0.

      The "N" or Negative flag will be set if the bit 7 result is a 1.
Otherwise it is cleared. The "Z" or Zero flag is set if all result bits are
zero,  otherwise, it is cleared.

       Flags
  N V E B D I Z C
  N - - - - - Z -


Arithmetic shifts, memory or accumulator, left or right         ASL ASR ASW

   ASL   Arithmetic shift left A or M      A<A<<1 or M<M<<1
   ASR   Arithmetic shift right A or M     A<A>>1 or M<M>>1
   ASW   Arithmetic shift left M (word)    Mx<Mw<<1

                                              Opcodes
  Addressing Mode               Abbrev.    ASL  ASR  ASW

  register (A)                              0A   43
  base page                     BP          06   44
  base page indexed X           BP,X        16   54
  absolute                      ABS         0E        CB
  absolute indexed X            ABS,X       1E

   Bytes   Cycles     Mode
     1       1      register (ASL)
     1       2      register (ASR)
     2       4      base page (byte) non-indexed, or indexed X
     3       5      absolute  non-indexed, or indexed X
     3       7      absolute (ASW)

      The ASL instructions shift a single byte of data in memory or the ac-
cumulator left (towards the most significant bit) one bit position. A 0 is
shifted into bit 0.

      The "N" or Negative bit will be set if the result bit 7 is (operand bit
6 was) a 1. Otherwise, it is cleared. The "Z" or Zero flag is set if ALL
result bits are zero. The "C" or Carry flag is set if the bit shifted out is
(operand bit 7 was) a 1. Otherwise, it is cleared.

      The ASR instructions shift a single byte of data in memory or the ac-
cumulator right (towards the least significant bit) one bit position. Since
this is an arithmetic shift, the sign of the operand will be maintained.

      The "N" or Negative bit will be set if bit 7 (operand and result) a 1.
Otherwise, it is cleared. The "Z" or Zero flag is set if ALL result bits are
zero. The "C" or Carry flag is set if the bit shifted out is (operand bit 0
was) a 1. Otherwise, it is cleared.

      The ASW instruction shifts a word (two bytes) of data in memory left
(towards the most significant bit) one bit position. A zero is shifted into
bit 0.
      The "N" or Negative bit will be set if the result bit 15 is (operand bit
14 was) a 1. Otherwise, it is cleared. The "Z" or Zero flag is set if ALL
result bits (both bytes) are zero. The "C" or Carry flag is set if the bit
shifted out is (operand bit 15 was) a 1. Otherwise, it is cleared.

       Flags
  N V E B D I Z C
  N - - - - - Z C



Bit test                                                                BIT

   (A.and.M)

  Addressing Mode               Abbrev.    Opcode

  immediate                     IMM         89
  base page                     BP          24
  base page indexed X           BP,X        34
  absolute                      ABS         2C
  absolute indexed X            ABS,X       3C

   Bytes   Cycles     Mode
     2       4      base page non-indexed, or indexed X
     3       5      absolute  non-indexed, or indexed X

      The BIT testing instructions perform a logical "and" between data
fetched from memory, and the accumulator. The result is not stored.

      The "N" flag will be set if bit 7 of the memory operand is a 1.
Otherwise it is cleared. The "V" flag will be set if bit 6 of the memory
operand is a 1. Otherwise, it is cleared. The "Z" or Zero flag will be set
if all result bits of the "and" operation are zero. Otherwise, it is cleared.

       Flags
  N V E B D I Z C
  7 6 - - - - Z -


Branch if memory bit reset or set                                   BBR BBS

             Opcode to test bit
          0   1   2   3   4   5   6   7

   BBR   0F  1F  2F  3F  4F  5F  6F  7F    Branch if bit reset
   BBS   8F  9F  AF  BF  CF  DF  EF  FF    Branch if bit set

   Bytes   Cycles     Mode
     3       4     base-page (test), byte-ralative (branch)

      The BBR instructions test a single bit within a Base-Page memory
location. If the bit is reset (or 0) the byte-offset relative branch
is taken.

      Similarly, the BBS performs a the same test. If the bit is set (or 1)
the branch is taken.

      The relative offset of the branch is referenced to the BBR or BBS
opcode+3, or the location after the three BBR or BBS instruction bytes.

        Flags
   N V E B D I Z C
   - - - - - - - -


Branch conditional or unconditional                     BCC BCS BEQ BMI BNE
                                                            BPL BRU BVC BVS

  Opcode  Opcode Byte  Opcode Word  Opcode
  Title    Relative     Relative    Purpose

    BCC       90           93       Branch if Carry Clear
    BCS       B0           B3       Branch if Carry Set
    BEQ       F0           F3       Branch if EQual (Z flag set)
    BMI       30           33       Branch if MInus (N flag set)
    BNE       D0           D3       Branch if Not Equal (Z flag clear)
    BPL       10           13       Branch if PLus  (N flag clear)
    BRU       80           83       BRanch Unconditional
    BVC       50           53       Branch if oVerflow Clear
    BVS       70           73       Branch if oVerflow Set

   Bytes   Cycles     Mode
     2       2     byte-relative
     3       3     word-relative

      All branches of this type are taken, if the condition indicated by
the opcode is true. All branch relative offsets are referenced to the
branch opcode location+2. This means that for byte-relative, the offset is
relative to the location after the two instruction bytes. For word-relative,
the offset is relative to the last of the three instruction bytes.

        Flags
   N V E B D I Z C
   - - - - - - - -


Break (force an interrupt)                                            BRK

   Bytes   Cycles     Mode        Opcode
     2       7      implied         00    (stack)<PC+1w,P SP<SP-2

      The BRK instruction causes the processor to enter the IRQ or Interrupt
ReQuest state. The program counter (now incremented by 2), bytes PCH and PCL,
and the processor status register P, are pushed onto the stack. (This causes
the stack pointer SP to be decremented by 3.) Then the program counter bytes
PCL and PCH are loaded from memory addresses FFFE and FFFF, respectively.

      The BRK differs from an externally generated interrupt request (IRQ) as
follows. The program counter value stored on the stack is PC+2, or the address
of the BRK opcode+2. On return from interrupt, the processor will return to
the BRK address+2, thus skipping the opcode byte, and a following "dummy" byte.
A normal IRQ will not add 2, so that a return will execute the interrupted
opcode. Also, when the P register is pushed onto the stack, the B or "break"
flag is set, to indicate that the interrupt was software generated. All outside
interrupts push P with the B flag cleared.

        Flags
   N V E B D I Z C
   - - - - - - - -


Branch to subroutine                                                  BSR

   Bytes   Cycles     Mode        Opcode
     3       5     word-relative    63    (stack)<PC+2w SP<SP-2

      The BSR Branch to SubRoutine instruction pushes the two program
counter bytes PCH and PCL onto the stack. It then adds the word-relative
signed offset to the program counter. The relative offset is referenced
to the address of the BSR opcode+2, hence, it is relative to the third
byte of the three-byte BSR instruction. The return address, on the stack,
also points to this address. This was done to make it compatible with the
RTS functionality, and to be consistant will other word-relative operations.


        Flags
   N V E B D I Z C
   - - - - - - - -


Clear processor status bits                           CLC CLD CLE CLI CLV

                                        Opcode  Cycles      Flags
                                                       N V E B D I Z C
   CLC   Clear the Carry bit              18      1    - - - - - - - R
   CLD   Clear the Decimal mode bit       D8      1    - - - - R - - -
   CLE   Clear stack Extend disable bit   02      2    - - R - - - - -
   CLI   Clear Interrupt disable bit      58      2    - - - - - R - -
   CLV   Clear the Oveflow bit            B8      1    - R - - - - - -

   Bytes    Mode
     1     implied

      All of the P register bit clear intructions are a single byte long.
Most of them require a single CPU cycle. The CLI and CLE require 2 cycles.
The purpose of extending the CLI to 2 cycles, is to enable an interrupt to
occur immediately, if one is pending. Interrupts cannot occur after single
cycle instructions.


Compare registers with memory                             CMP CPX CPY CPZ

   CMP   Compare accumulator with memory  (A-M)
   CPX   Compare index X with memory      (X-M)
   CPY   Compare index Y with memory      (Y-M)
   CPZ   Compare index Z with memory      (Z-M)

                                                Opcodes
  Addressing Mode               Abbrev.    CMP  CPX  CPY  CPZ

  immediate                     IMM         C9   E0   C0   C2
  base page                     BP          C5   E4   C4   D4
  base page indexed X           BP,X        D5
  absolute                      ABS         CD   EC   CC   DC
  absolute indexed X            ABS,X       DD
  absolute indexed Y            ABS,Y       D9
  base page indexed indirect X  (BP,X)      C1
  base page indirect indexed Y  (BP),Y      D1
  base page indirect indexed Z  (BP),Z      D2

   Bytes   Cycles     Mode
     2       2      immediate
     2       3      base page non-indexed, or indexed X or Y
     3       4      absolute  non-indexed, or indexed X or Y
     2       5      base page indexed indirect X, or indirect indexed Y or Z

      Compares are performed by subtracting a value in memory from the
register being tested. The results are not stored in any register, except
the following status flags are updated.

      The "N" or Negative flag will be set if the result is negative (assuming
signed operands), otherwise it is cleared. The "Z" or Zero flag is set if the
result is zero, otherwise it is cleared. The "C" or carry flag is set if the
unsigned register value is greater than or equal to the unsigned memory value.

       Flags
  N V E B D I Z C
  N - - - - - Z C


Decrement Registers or Memory                         DEC DEW DEX DEY DEZ

   DEC   Decrement accumulator or memory    A<A+1 or M<M+1
   DEW   Decrement a memory word            Mw<Mw+1
   DEX   Decrement index X                  X<X+1
   DEY   Decrement index Y                  Y<Y+1
   DEZ   Decrement index Z                  Z<Z+1

                                                   Opcodes
  Addressing Mode               Abbrev.    DEC  DEW  DEX  DEY  DEZ

  register (DEC implies accumulator)        3A        CA   88   3B
  base page                     BP          C6   C3
  base page indexed X           BP,X        D6
  absolute                      ABS         CE
  absolute indexed X            ABS,X       DE

   Bytes   Cycles     Mode
     1       1      register
     2       4      base page (byte) non-indexed, or indexed X
     2       6      base page (word)
     3       5      absolute  non-indexed, or indexed X

      The DEC (accumulator), DEX, DEY, and DEZ instructions are single-byte,
single-cycle, and decrement (or subtract 1 from) the specified register.

      The DEC (memory), and DEW (memory) instructions decrement a byte or
word, respectively, in memory.

      The "N" or "negative" flag is set if the result value is negative.
Otherwise, it is cleared. The "Z" or "zero" flag is set if the result of the
decrement is zero. Otherwise, it is cleared.

       Flags
  N V E B D I Z C
  N - - - - - Z -


End-of-Mapping sequence							 EOM

  Addressing Mode  Opcode  Bytes  Cycles

    implied          EA      1      1


      The EOM end-of-map instruction has no effect, unless used following
a MAP opcode. The MAP opcode inhibits all interrupts. (NMI and IRQ). The
EOM opcode removes this interrupt inhibit. Any other time, this instruction
is purely a NOP no-operation. (see NOP)

       Flags
  N V E B D I Z C
  - - - - - - - -

Exclusive OR accumulator logically with memory                           EOR

A=A.or.M.and..not.(A.and.M)

  Addressing Mode               Abbrev.    Opcode

  immediate                     IMM         49
  base page                     BP          45
  base page indexed X           BP,X        55
  absolute                      ABS         4D
  absolute indexed X            ABS,X       5D
  absolute indexed Y            ABS,Y       59
  base page indexed indirect X  (BP,X)      41
  base page indirect indexed Y  (BP),Y      51
  base page indirect indexed Z  (BP),Z      52

   Bytes   Cycles     Mode
     2       2      immediate
     2       3      base page non-indexed, or indexed X or Y
     3       4      absolute  non-indexed, or indexed X or Y
     2       5      base page indexed indirect X, or indirect indexed Y or Z

      The EOR instructions perform an "exclusive or" between bits fetched
from memory and the accumulator bits. The results are then stored in the
accumulator. For each accumulator or corresponding memory bit that are
different (one 1, and one 0) the result is a 1. Otherwise it is 0.

      The "N" or Negative flag will be set if the bit 7 result is a 1.
Otherwise it is cleared. The "Z" or Zero flag is set if all result bits are
zero,  otherwise, it is cleared.

       Flags
  N V E B D I Z C
  N - - - - - Z -


Increment Registers or Memory                         INC INW INX INY INZ

   INC   Increment accumulator or memory   A<A+1 or M<M+1
   INW   Increment a memory word           Mw<Mw+1
   INX   Increment index X                 X<X+1
   INY   Increment index Y                 Y<Y+1
   INZ   Increment index Z                 Z<Z+1

                                                   Opcodes
  Addressing Mode               Abbrev.    INC  INW  INX  INY  INZ

  register (INC impliess INC A)             1A        E8   C8   1B
  base page                     BP          E6   E3
  base page indexed X           BP,X        F6
  absolute                      ABS         EE
  absolute indexed X            ABS,X       FE

   Bytes   Cycles     Mode
     1       1      register
     2       4      base page (byte) non-indexed, or indexed X
     2       6      base page (word)
     3       5      absolute  non-indexed, or indexed X

      The INC (accumulator), INX, INY, and INZ instructions are single-byte,
single-cycle, and increment (or add 1 to) the specified register.

      The INC (memory), and INW (memory) instructions increment a byte or
word, respectively, in memory.

      The "N" or "negative" flag is set if the result value is negative.
Otherwise, it is cleared. The "Z" or "zero" flag is set if the result of the
increment is zero. Otherwise, it is cleared.

       Flags
  N V E B D I Z C
  N - - - - - Z -


Jump to new location                                                  JMP


  Addressing Mode               Abbrev.   Opcode  bytes  cycles

  absolute                      ABS         4C      3      3
  absolute indirect             (ABS)       6C      3      5
  absolute indexed indirect X   (ABS,X)     7C      3      5

       Flags
  N V E B D I Z C
  - - - - - - - -


Jump to subroutine                                                  JSR

  Addressing Mode               Abbrev.   Opcode  bytes  cycles

  absolute                      ABS         20      3      5
  absolute indirect             (ABS)       22      3      7
  absolute indexed indirect X   (ABS,X)     23      3      7

      The JSR Jump to SubRoutine instruction pushes the two program counter
bytes PCH and PCL onto the stack. It then loads the program counter with the
new address. The return address, stored on the stack, is actually the address
of the JSR opcode+2, or is pointing to the third byte of the three-byte JSR 
instruction.

       Flags
  N V E B D I Z C
  - - - - - - - -


Load registers                                            LDA LDX LDY LDZ

   LDA   Load Accumulator from memory   A<M
   LDX   Load index X from memory       X<M
   LDY   Load index Y from memory       Y<M
   LDZ   Load index Z from memory       Z<M

  Addressing Mode               Abbrev.    LDA  LDX  LDY  LDZ

  immediate                     IMM         A9   A2   A0   A3
  base page                     BP          A5   A6   A4
  base page indexed X           BP,X        B5        B4
  base page indexed Y           BP,Y             B6
  absolute                      ABS         AD   AE   AC   AB
  absolute indexed X            ABS,X       BD        BC   BB
  absolute indexed Y            ABS,Y       B9   BE
  base page indexed indirect X  (BP,X)      A1
  base page indirect indexed Y  (BP),Y      B1
  base page indirect indexed Z  (BP),Z      B2
  stack vector indir indexed Y  (d,SP),Y    E2

   Bytes   Cycles     Mode
     2       2      immediate
     2       3      base page non-indexed, or indexed X or Y
     3       4      absolute  non-indexed, or indexed X or Y
     2       5      base page indexed indirect X, or indirect indexed Y or Z
     2       6      stack vector indirect indexed Y

      These instructions load the specified register from memory. The "N"
or Negative flag will be set if the bit 7 loaded is a 1. Otherwise it is
cleared. The "Z" or Zero flag is set if all bits loaded are zero, otherwise,
it is cleared.

       Flags
  N V E B D I Z C
  7 - - - - - Z -


Logical shifts, memory or accumulator, right                          LSR

  A<A>>1 or M<M>>1
                                         Opcodes
  Addressing Mode               Abbrev.    LSR

  register (A)                              4A
  base page                     BP          46
  base page indexed X           BP,X        56
  absolute                      ABS         4E
  absolute indexed X            ABS,X       5E

   Bytes   Cycles     Mode
     1       1      register
     2       4      base page non-indexed, or indexed X
     3       5      absolute  non-indexed, or indexed X

      The LSR instructions shift a single byte of data in memory or the ac-
cumulator right (towards the least significant bit) one bit position. A 0 is
shifted into bit 7 (the sign bit).

      The "N" or Negative bit will be cleared. The "Z" or Zero flag is set if
ALL result bits are zero. The "C" or Carry flag is set if the bit shifted out
is (operand bit 0 was) a 1. Otherwise, it is cleared.

       Flags
  N V E B D I Z C
  R - - - - - Z C


Change the Memory Mapper Context                                         MAP

  Addressing Mode     Opcode  bytes  cycles

   implied              5C      4      4

      The MAP instruction inhibits all interrupts, and transfers the contents
of the A, X, Y, and Z registers to the mapper registers.


        Flags
   N V E B D I Z C
   - - - - - - - -

Negate (twos complement) accumulator                                     NEG

A=-A

  Addressing Mode  Opcode  Bytes  Cycles

    implied          42      1      2

      The NEG or "negate" instruction performs a two's-complement inversion
of the data in the accumulator. For example, 1 becomes -1, -5 becomes 5, etc.
The same can be achieved by subtracting A from zero.

      The "N" or Negative flag will be set if the accumulator bit 7 becomes
a 1. Otherwise it is cleared. The "Z" or Zero flag is set if the accumulator
is (and was) zero.

       Flags
  N V E B D I Z C
  N - - - - - Z -


No-operation                                                             NOP

  Addressing Mode  Opcode  Bytes  Cycles

    implied          EA      1      1


      The NOP no-operation instruction has no effect, unless used following
a MAP opcode. Then its is interpreted as a EOM end-of-map instruction.
(See EOM)

       Flags
  N V E B D I Z C
  - - - - - - - -


Or memory logically with accumulator                                     ORA

A=A.or.M

  Addressing Mode               Abbrev.    Opcode

  immediate                     IMM         09
  base page                     BP          05
  base page indexed X           BP,X        15
  absolute                      ABS         0D
  absolute indexed X            ABS,X       1D
  absolute indexed Y            ABS,Y       19
  base page indexed indirect X  (BP,X)      01
  base page indirect indexed Y  (BP),Y      11
  base page indirect indexed Z  (BP),Z      12

   Bytes   Cycles     Mode
     2       2      immediate
     2       3      base page non-indexed, or indexed X or Y
     3       4      absolute  non-indexed, or indexed X or Y
     2       5      base page indexed indirect X, or indirect indexed Y or Z

      The ORA instructions perform a logical "or" between data bits fetched
from memory and the accumulator bits. The results are then stored in the
accumulator. For either accumulator or corresponding memory bit that is a
logical 1's, the result is a 1. Otherwise it is 0.

      The "N" or Negative flag will be set if the bit 7 result is a 1.
Otherwise it is cleared. The "Z" or Zero flag is set if all result bits are
zero,  otherwise, it is cleared.

       Flags
  N V E B D I Z C
  N - - - - - Z -


Pull register data from stack                         PLA PLP PLX PLY PLZ

                                         Opcode

   PLA   Pull Accumulator from stack       68
   PLX   Pull index X from stack           FA
   PLY   Pull index Y from stack           7A
   PLZ   Pull index Z from stack           FB
   PLP   Pull Processor status from stack  28

   Bytes   Cycles     Mode
     1       3      register

      The Pull register operations, first, increment the stack pointer SP,
and then, load the specified register with data from the stack.

      Except in the case of PLP, the "N" or Negative flag will be set if the
bit 7 loaded is a 1. Otherwise it is cleared. The "Z" or Zero flag is set if
all bits loaded are zero, otherwise, it is cleared.

      In the case of PLP, all processor flags (P register bits) will be loaded
from the stack, except the "B" or "break" flag, which is always a 1, and the
"E" or "stack pointer Extend disable" flag, which can only be set by SEE, or
cleared by CLE instructions.

      Flags
  N V E B D I Z C
  N - - - - - Z -  (except PLP)
  7 6 - - 3 2 1 0  (PLP only)


Push registers or data onto stack                 PHA PHP PHW PHX PHY PHZ

   PHA   Push Accumulator onto stack
   PHP   Push Processor status onto stack
   PHW   Push a word from memory onto stack
   PHX   Push index X onto stack
   PHY   Push index Y onto stack
   PHZ   Push index Z onto stack

                                                Opcodes
  Addressing Mode               Abbrev.    PHA  PHP  PHW  PHX  PHY  PHZ

  register                                  48   08        DA   5A   DB
  word immediate                IMMw                  F4
  word absolute                 ABSw                  FC

   Bytes   Cycles     Mode
     1       3      register
     3       5      word immediate
     3       7      word absolute

      These instructions push either the contents of a register onto the
stack, or push two bytes of data from memory (PHW) onto the stack. If a
register is pushed, the stack pointer will decrement a single address. If
a word from memory is pushed, the stack pointer will decrement by 2. No
flags are changed.

       Flags
  N V E B D I Z C
  - - - - - - - -

Reset memory bits                                                      RMB

M=M.and.-bit

           Opcode to reset bit
      0   1   2   3   4   5   6   7

     07  17  27  37  47  57  67  77

   Bytes   Cycles     Mode
     2       4     base-page

      These instructions reset a single bit in base-page memory, as specified
by the opcode. No  flags are modified.

        Flags
   N V E B D I Z C
   - - - - - - - -


Rotate memory or accumulator, left or right                    ROL ROR ROW

   ROL   Rotate memory or accumulator left throught carry
   ROR   Rotate memory or accumulator right throught carry
   ROW   Rotate memory (word) left throught carry

                                           Opcodes
  Addressing Mode               Abbrev.    ROL  ROR  ROW

  register (A)                              2A   6A
  base page                     BP          26   66
  base page indexed X           BP,X        36   76
  absolute                      ABS         2E   6E   EB
  absolute indexed X            ABS,X       3E   7E

   Bytes   Cycles     Mode
     1       1      register
     2       4      base page (byte) non-indexed, or indexed X
     3       5      absolute  non-indexed, or indexed X
     2       6      absolute (word)

      The ROL instructions shift a single byte of data in memory or the ac-
cumulator left (towards the most significant bit) one bit position. The state
of the "C" or "carry" flag is shifted into bit 0.

      The "N" or Negative bit will be set if the result bit 7 is (operand bit
6 was) a 1. Otherwise, it is cleared. The "Z" or Zero flag is set if ALL
result bits are zero. The "C" or Carry flag is set if the bit shifted out is
(operand bit 7 was) a 1. Otherwise, it is cleared.

      The ROR instructions shift a single byte of data in memory or the ac-
cumulator right (towards the least significant bit) one bit position. The
state of the "C" or "carry" flag is shifted into bit 7.

      The "N" or Negative bit will be set if bit 7 is (carry was) a 1.
Otherwise, it is cleared. The "Z" or Zero flag is set if ALL result bits are
zero. The "C" or Carry flag is set if the bit shifted out is (operand bit 0
was) a 1. Otherwise, it is cleared.

      The ROW instruction shifts a word (two bytes) of data in memory left
(towards the most significant bit) one bit position. The stateof the "C" or
"carry" flag is shifted into bit 0.

      The "N" or Negative bit will be set if the result bit 15 is (operand bit
14 was) a 1. Otherwise, it is cleared. The "Z" or Zero flag is set if ALL
result bits (both bytes) are zero. The "C" or Carry flag is set if the bit
shifted out is (operand bit 15 was) a 1. Otherwise, it is cleared.

       Flags
  N V E B D I Z C
  N - - - - - Z C


Return from BRK, interrupt, kernal, or subroutine                RTI RTN RTS

Operation    description         Opcode  bytes  cycles  

   RTI    Return from interrupt    40      1      5     P,PCw<(SP),SP<SP+3
   RTN #n Return from kernal       62      2      7     PCw<(SP)+1,SP<SP+2+N
   RTS    Return from subroutine   60      1      4     PCw<(SP)+1,SP<SP+2

      The RTI or ReTurn from Interrupt instruction pulls P register data
and a return address into program counter bytes PCL and PCH from the stack.
The stack pointer SP is resultantly incremented by 3. Execution continues
at the address recovered from the stack.

       Flags
  N V E B D I Z C
  7 6 - - 3 2 1 0 (RTI only)

      The RTS or ReTurn from Subroutine instruction pulls a return address
into program counter bytes PCL and PCH from the stack. The stack pointer SP
is resultantly incremented by 2. Execution continues at the address recovered
 + 1, since BSR and JSR instructions set the return address one byte short of
the desire return address.

      The RTN or ReTurn from kerNal subroutine is similar to RTS, except that
it contains an immediate parameter N indicating how many extra bytes to discard
from the stack. This is useful for returning from subroutines which have
arguments passed to them on the stack. The stack pointer SP is incremented by
2 + N, instead of by 2, as in RTS.

       Flags
  N V E B D I Z C
  - - - - - - - - (RTN and RTS)
  7 6 - - 3 2 1 0 (RTI)

Subtract memory from accumulator with borrow                             SBC

A=A-M+C-1

  Addressing Mode               Abbrev.    Opcode

  immediate                     IMM         E9
  base page                     BP          E5
  base page indexed X           BP,X        F5
  absolute                      ABS         ED
  absolute indexed X            ABS,X       FD
  absolute indexed Y            ABS,Y       F9
  base page indexed indirect X  (BP,X)      E1
  base page indirect indexed Y  (BP),Y      F1
  base page indirect indexed Z  (BP),Z      F2

   Bytes   Cycles     Mode
     2       2      immediate
     2       3      base page non-indexed, or indexed X or Y
     3       4      absolute  non-indexed, or indexed X or Y
     2       5      base page indexed indirect X, or indirect indexed Y or Z

      The SBC instructions subtract data fetched from memory from the contents
of the accumulator, assuming the "C" or "carry" flag was set. If "C" was clear,
an additional count is subtracted. The results of the subtract is stored in
the accumulator. If the "D" or Decimal Mode flag, in the processor status regis-
ter, then a Binary Coded Decimal (BCD) subtract is performed.

      The "N" or Negative flag will be set if the difference is negative,
otherwise it is cleared. The "V" or Overflow flag will be set if the sign
of the difference is different from the sign of both operands, indicating a
signed overflow. Otherwise, it is cleared. The "Z" or Zero flag is set if
the difference (stored into the accumulator) is zero, otherwise, it is cleared.
The "C" or carry is set if the unsigned minuend (in the accumulator) is greater-
than or equal-to the unsigned subtrahend (in memory). Otherwise, it is cleared.

       Flags
  N V E B D I Z C
  N V - - - - Z C


Set processor status bits                                 SEC SED SEE SEI

                                        Opcode  Cycles      Flags
                                                       N V E B D I Z C
   SEC   Set the Carry bit                38      1    - - - - - - - S
   SED   Set the Decimal mode bit         F8      1    - - - - S - - -
   SEE   Set stack Extend disable bit     03      2    - - S - - - - -
   SEI   Set Interrupt disable bit        78      2    - - - - - S - -

   Bytes    Mode
     1     implied

      All of the P register bit set intructions are a single byte long.
Most of them require a single CPU cycle. The SEE and SEI require 2 cycles.


Set memory bits                                                      SMB

M=M.or.bit

           Opcode to set bit
      0   1   2   3   4   5   6   7

     87  97  A7  B7  C7  D7  E7  F7

   Bytes   Cycles     Mode
     2       4     base-page

      These instructions set a single bit in base-page memory, as specified
by the opcode. No flags are modified.

        Flags
   N V E B D I Z C
   - - - - - - - -


Store registers                                           STA STX STY STZ

   STA   Store Accumulator to memory   M<A
   STX   Store index X to memory       M<X
   STY   Store index Y to memory       M<Y
   STZ   Store index Z to memory       M<Z

                                                Opcodes
  Addressing Mode               Abbrev.    STA  STX  STY  STZ

  base page                     BP          85   86   84   64
  base page indexed X           BP,X        95        94   74
  base page indexed Y           BP,Y             96
  absolute                      ABS         8D   8E   8C   9C
  absolute indexed X            ABS,X       9D        8B   9E
  absolute indexed Y            ABS,Y       99   9B
  base page indexed indirect X  (BP,X)      81
  base page indirect indexed Y  (BP),Y      91
  base page indirect indexed Z  (BP),Z      92
  stack vector indir indexed Y  (d,SP),Y    82

   Bytes   Cycles     Mode
     2       3      base page non-indexed, or indexed X or Y
     3       4      absolute  non-indexed, or indexed X or Y
     2       5      base page indexed indirect X, or indirect indexed Y or Z
     2       6      stack vector indirect indexed Y

      These instructions store the specified register to memory. No flags
are affected.

       Flags
  N V E B D I Z C
  - - - - - - - -


Transfers (between registers)                             TAB TAX TAY TAZ
                                                          TBA TSX TSY TXA
                                                          TXS TYA TYS TZA
   Operation          Flags         Transfer
   Symbol  Code   N V E B D I Z C   from            to

    TAB     5B    - - - - - - - -   accumulator     base page reg
    TAX     AA    N - - - - - Z -   accumulator     index X reg
    TAY     A8    N - - - - - Z -   accumulator     index Y reg
    TAZ     4B    N - - - - - Z -   accumulator     index Z reg
    TBA     7B    N - - - - - Z -   base page reg   accumulator
    TSX     BA    N - - - - - Z -   stack ptr low   index X reg
    TSY     0B    N - - - - - Z -   stack ptr high  index Y reg
    TXA     8A    N - - - - - Z -   index X reg     accumulator
    TXS     9A    - - - - - - - -   index X reg     stack ptr low
    TYA     98    N - - - - - Z -   index Y reg     accumulator
    TYS     2B    - - - - - - - -   index Y reg     stack ptr high
    TZA     6B    N - - - - - Z -   index Z reg     accumulator

      These instructions transfer the contents of the specified source register
to the specified destination register. Any transfer to A, X, Y, or Z will
affect the flags as follows. The "N" or "negative" flag will be set if the
value moved is negative (bit 7 set), otherwise, it is cleared. The "Z" or
"zero" flag will be set if the value moved is zero (all bits 0), otherwise, it
is cleared. Any transfer to SPL or SPH will not alter any flags.

	************************************************************
	*			WARNING				   *
	*							   *
	*  If you are using Non-Maskable-Interrupts, or Interrupt  *
	*  Request is enabled, and you want to change BOTH stack   *
	*  pointer bytes, do not put any code between the TXS and  *
	*  TYS opcodes. Taking this precaution will prevent any    *
	*  interrupts from occuring between the setting of the two *
	*  stack pointer bytes, causing a potential for writing    *
	*  stack data to an unwanted area.			   *
	************************************************************

   Bytes   Cycles     Mode
     1       1      register

 
Test and reset or set memory bits                                 TRB TSB

   TRB   Test and reset memory bits with accumulator  (M.or.A),M<M.and.-A
   TSB   Test and set  memory bits with accumulator   (M.or.A),M<M.or.A

                                            Opcodes
  Addressing Mode               Abbrev.    TRB  TSB

  base page                     BP          14   04
  absolute                      ABS         1C   0C

      These instructions test and set or reset bits in memory, using the
accumulator for both a test mask, and a set or reset mask. First, a logical
AND is performed between memory and the accumulator. The "Z" or "zero" flag
is set if all bits of the result of the AND are zero. Otherwise it is reset.

      The TSB then performs a logical OR between the bits of the accumulator
and the bits in memory, storing the result back into memory.

      The TRB, instead, performs a logical AND between the inverted bits of
the accumulator and the bits in memory, storing the result back into memory.


       Flags
  N V E B D I Z C
  - - - - - - Z -

			Opcode Table

(0[1 K  0    1    2    3    4    5    6    7    8    9    A    B    C    D    E    F
lqqqqwqqqqwqqqwwqqqwwqqqqwqqqqwqqqqwqqqqwqqqqwqqqqwqqqqwqqqwwqqqqwqqqqwqqqqwqqqqk
xBRK xORA xCLEmuSEEmuTSB xORA xASL xRMB0xPHP xORA xASL xTSYmuTSB xORA xASL xBBR0x
x    xINDXx    x    xZP  xZP  xZP  xZP  x    xIMM x    x    xABS xABS xABS xZP  x0
tqqqqnqqqqnqqqqnqqqwnqqqqnqqqqnqqqqnqqqqnqqqqnqqqqnqqqqnqqqwnqqqqnqqqqnqqqqnqqqqu
xBPL xORA xORA xBPLmuTRB xORA xASL xRMB1xCLC xORA xINC xINZmuTRB xORA xASL xBBR1x
xREL xINDYxINDZxWRELxZPX xZPX xZPX xZP  x    xABSYx    x    xABS xABSXxABSXxZP  x1
tqqqqnqqqqnqqqwnqqqwnqqqqnqqqqnqqqqnqqqqnqqqqnqqqqnqqqqnqqqwnqqqqnqqqqnqqqqnqqqqu
xJSR xAND xJSRmuJSRmuBIT xAND xROL xRMB2xPLP xAND xROL xTYSmuBIT xAND xROL xBBR2x
xABS xINDXxIND xINDXxZP  xZP  xZP  xZP  x    xIMM x    x    xABS xABS xABS xZP  x2
tqqqqnqqqqnqqqqnqqqwnqqqqnqqqqnqqqqnqqqqnqqqqnqqqqnqqqqnqqqwnqqqqnqqqqnqqqqnqqqqu
xBMI xAND xAND xBMImuBIT xAND xROL xRMB3xSEC xAND xDEC xDEZmuBIT xAND xROL xBBR3x
xREL xINDYxINDZxWRELxZPX xZPX xZPX xZP  x    xABSYx    x    xABSXxABSXxABSXxZP  x3
tqqqqnqqqqnqqqwnqqqwnqqqwnqqqqnqqqqnqqqqnqqqqnqqqqnqqqqnqqqwnqqqqnqqqqnqqqqnqqqqu
xRTI xEOR xNEGmuASRmuASRmuEOR xLSR xRMB4xPHA xEOR xLSR xTAZmuJMP xEOR xLSR xBBR4x
x    xINDXx    x    xZP  xZP  xZP  xZP  x    xIMM x    x    xABS xABS xABS xZP  x4
tqqqqnqqqqnqqqqnqqqwnqqqwnqqqqnqqqqnqqqqnqqqqnqqqqnqqqqnqqqwnqqqwnqqqqnqqqqnqqqqu
xBVC xEOR xEOR xBVCmuASRmuEOR xLSR xRMB5xCLI xEOR xPHY xTABmuMAPmuEOR xLSR xBBR5x
xREL xINDYxINDZxWRELxZPX xZPX xZPX xZP  x    xABSYx    x    x    xABSXxABSXxZP  x5
tqqqqnqqqqnqqqwnqqqwnqqqqnqqqqnqqqqnqqqqnqqqqnqqqqnqqqqnqqqwnqqqqnqqqqnqqqqnqqqqu
xRTS xADC xRTNmuBSRmuSTZ xADC xROR xRMB6xPLA xADC xROR xTZAmuJMP xADC xROR xBBR6x
x    xINDXx    xWRELxZP  xZP  xZP  xZP  x    xIMM x    x    xIND xABS xABS xZP  x6
tqqqqnqqqqnqqqqnqqqwnqqqqnqqqqnqqqqnqqqqnqqqqnqqqqnqqqqnqqqwnqqqqnqqqqnqqqqnqqqqu
xBVS xADC xADC xBVSmuSTZ xADC xROR xRMB7xSEI xADC xPLY xTBAmuJMP xADC xROR xBBR7x
xREL xINDYxINDZxWRELxZPX xZPX xZPX xZP  x    xABSYx    x    xINDXxABSXxABSXxZP  x7
tqqqqnqqqqnqqqwnqqqwnqqqqnqqqqnqqqqnqqqqnqqqqnqqqqnqqqqnqqqwnqqqqnqqqqnqqqqnqqqqu
xBRU xSTA xSTAmuBRUmuSTY xSTA xSTX xSMB0xDEY xBIT xTXA xSTYmuSTY xSTA xSTX xBBS0x
xREL xINDXxIDSPxWRELxZP  xZP  xZP  xZP  x    xIMM x    xABSXxABS xABS xABS xZP  x8
tqqqqnqqqqnqqqqnqqqwnqqqqnqqqqnqqqqnqqqqnqqqqnqqqqnqqqqnqqqwnqqqqnqqqqnqqqqnqqqqu
xBCC xSTA xSTA xBCCmuSTY xSTA xSTX xSMB1xTYA xSTA xTXS xSTXmuSTZ xSTA xSTZ xBBS1x
xREL xINDYxINDZxWRELxZPX xZPX xZPY xZP  x    xABSYx    xABSYxABS xABSXxABSXxZP  x9
tqqqqnqqqqnqqqqnqqqwnqqqqnqqqqnqqqqnqqqqnqqqqnqqqqnqqqqnqqqwnqqqqnqqqqnqqqqnqqqqu
xLDY xLDA xLDX xLDZmuLDY xLDA xLDX xSMB2xTAY xLDA xTAX xLDZmuLDY xLDA xLDX xBBS2x
xIMM xINDXxIMM xIMM xZP  xZP  xZP  xZP  x    xIMM x    xABS xABS xABS xABS xZP  xA
tqqqqnqqqqnqqqqnqqqwnqqqqnqqqqnqqqqnqqqqnqqqqnqqqqnqqqqnqqqwnqqqqnqqqqnqqqqnqqqqu
xBCS xLDA xLDA xBCSmuLDY xLDA xLDX xSMB3xCLV xLDA xTSX xLDZmuLDY xLDA xLDX xBBS3x
xREL xINDYxINDZxWRELxZPX xZPX xZPY xZP  x    xABSYx    xABSXxABSXxABSXxABSYxZP  xB
tqqqqnqqqqnqqqwnqqqwnqqqqnqqqqnqqqqnqqqqnqqqqnqqqqnqqqqnqqqwnqqqqnqqqqnqqqqnqqqqu
xCPY xCMP xCPZmuDEWmuCPY xCMP xDEC xSMB4xINY xCMP xDEX xASWmuCPY xCMP xDEC xBBS4x
xIMM xINDXxIMM xZP  xZP  xZP  xZP  xZP  x    xIMM x    xABS xABS xABS xABS xZP  xC
tqqqqnqqqqnqqqqnqqqwnqqqwnqqqqnqqqqnqqqqnqqqqnqqqqnqqqqnqqqwnqqqwnqqqqnqqqqnqqqqu
xBNE xCMP xCMP xBNEmuCPZmuCMP xDEC xSMB5xCLD xCMP xPHX xPHZmuCPZmuCMP xDEC xBBS5x
xREL xINDYxINDZxWRELxZP  xZPX xZPX xZP  x    xABSYx    x    xABS xABSXxABSXxZP  xD
tqqqqnqqqqnqqqwnqqqwnqqqqnqqqqnqqqqnqqqqnqqqqnqqqqnqqqqnqqqwnqqqqnqqqqnqqqqnqqqqu
xCPX xSBC xLDAmuINWmuCPX xSBC xINC xSMB6xINX xSBC xEOM xROWmuCPX xSBC xINC xBBS6x
xIMM xINDXxIDSPxZP  xZP  xZP  xZP  xZP  x    xIMM xNOP xABS xABS xABS xABS xZP  xE
tqqqqnqqqqnqqqqnqqqwnqqqwnqqqqnqqqqnqqqqnqqqqnqqqqnqqqqnqqqwnqqqwnqqqqnqqqqnqqqqu
xBEQ xSBC xSBC xBEQmuPHDmuSBC xINC xSMB7xSED xSBC xPLX xPLZmuPHDmuSBC xINC xBBS7x
xREL xINDYxINDZxWRELxIMM xZPX xZPX xZP  x    xABSYx    x    xABS xABSXxABSXxZP  xF
mqqqqvqqqqvqqqqvqqqqvqqqqvqqqqvqqqqvqqqqvqqqqvqqqqvqqqqvqqqqvqqqqvqqqqvqqqqvqqqqj
(B[0 K
