
		    C64DX DMAGIC (F018B) DMA COPROCESSOR 

INTRODUCTION

The C64DX DMAgic chip is a command-list DMA (Direct Memory Access) coprocessor
with integrated blitter capabilities.  It is used to copy or fill linear or
rectangular areas of memory much faster than is possible using the CPU. 
DMAgic can perform logical operations and shifts of binary data, and it is
particularly well suited to graphics operations.  It has absolute address
access to the entire eight megabyte address space of the C64DX, and translated
address access to VIC graphics memory using simple X-Y coordinates.

A DMA/blitter coprocessor can be used for many different types of operations
in a computer system such as the C64DX, and DMAgic is very versatile in this
regard.  Although each of these uses may require significantly different
command specifications, DMAgic is very easy to program and use.

Throughout this manual the term "DMAT" will refer to any DMAgic Transfer
operation.  A DMAT is prepared by creating a 12-byte DMAT instruction list
anywhere in the first 1MB of RAM.  A DMAT is executed automatically after
writing the address of the DMAT instruction list to DMAgic's LP/trigger
(List Pointer) register.

A DMAT list can be prepared in advance and executed at any time by pointing
the DMAgic chip to it.  Optionally, DMAT lists can be chained together and
the collection of lists executed with a single trigger.  Additionally, DMATs
can be interruptable/continuable to yield to processor or system interrupts.

In general, a DMAT instruction list consists of a Command specifying the type
of operation to be performed, a Count of the number of memory bytes to be
transferred, and the Source and Destination addresses of the data to be
transferred.  When the address of the instruction list is written to the
LP/trigger register, DMAgic temporarily takes control of the system from the
CPU, automatically loads its internal registers from the instruction list, and
proceeds to execute the specified DMAT operation.  The completion status of
the operation can be read from DMAgic's status register, and (optionally)
interrupted operations can be resumed or canceled.

DMAT EXTERNAL REGISTERS

Externally, DMAgic appears as four registers located in I/O space at $D700.
There are three write-only address registers and one R/W status register.
The order in which the address/trigger registers are written is very important,
so as to avoid inadvertantly triggering a DMA operation with an invalid list
pointer.  The usual procedure is:

	1. Write the number of the list's bank (0-15) to LP_BNK.
	2. Write the high byte of the list's address to LP_HI.
	3. Write the low byte of the list's address to LP_LO/TRIGGER.

As soon as the LP_LO/TRIGGER register is written to, the CPU is suspended and
the DMA operation commences.  When the DMA operation is complete, the CPU
resumes processing with the very next instruction following the write cycle.
The contents of the instruction list, and the DMAgic LP registers, are
unchanged.  The very same DMA operation could be performed again by performing
step 3 above only.

In the case where a DMAT is made interruptable and an interrupt (IRQ or NMI)
were to occur, DMAgic would enter a "spin" state, and release the bus to the
CPU.  Upon completion of its interrupt service routine, the CPU would signal
DMAgic that it can resume the DMAT by reading the STATUS register, or cancel
the unfinished DMAT by writing to the STATUS register (writing to the TRIGGER
register would effectively cancel the pending DMAT and start a new one).

The following code illustrates a generic DMAT call for a previously prepared
DMAT instruction list:


		LDA #bank	;The bank (0-15) where the list is
		STA $D702
		LDA #>address	;The high byte of the list address
		STA $D701
		LDA #<address
		STA $D700	;The low byte of the list address
				;This also triggers the DMA operation,
				;and the CPU is suspended

	loop:	BIT $D703	;Check status (in case IRQ/NMI enabled)
		BMI loop	;	busy


DMAT INTERNAL REGISTERS

A DMAT operation is specified by a twelve-byte command spec called a DMAT list.
Upon writing to the LP_LO/TRIGGER register ($D700), the coprocessor will fetch
the DMAT list and initialize its internal registers according to the contents
of the list.  The DMAT list consists of a primary COMMAND byte, two COUNT
bytes, three SOURCE Address bytes, three DESTINATION Address bytes, a secondary
COMMAND byte, and two MODULO/MODE bytes, as shown in the register description.

The entire list must be completely specified; unused (unrequired) parameters
must be set to zero.  Many of the internal registers have different functions,
dependent upon the command mode(s) selected.

DMAgic OPERATING MODES

The primary command byte CMD_LO specifies what kind of DMAT operation is to
be performed:

*  COPY	- Copy a block of memory to another area in memory.

*  MIX	- Perform a boolean minterm mix of a source block of memory with a 
	  destination block of memory.

*  SWAP	- Exchange the contents of two blocks of memory.

*  FILL	- Fill a block of memory with a source byte.

Also located in the CMD_LO byte are various control parameters for the command.
An S_DIR and D_DIR bit provide independant direction control for both the
source and destination address pointers.  Transfers may be ascending (UP),
or descending (DOWN), or mixed.  The TEST bit can be set to tell DMAgic to test
the memory blocks without WRITING a result, and to quit early if a collision or
verify error is detected.  If the CHAIN bit is set, DMAgic will fetch and
execute the next twelve-byte DMAT list in memory upon completion of this DMAT.
If the INT bit is set, DMAgic will suspend this transfer if either an IRQ or
NMI system interrupt occurs.  The operation will be continued following a READ
of the STATUS register ($D703).

The secondary command byte CMD_HI specifies the addressing mode DMAgic is to
use during the DMAT operation.  There are four addressing modes: LINEAR MODE,
MODULO MODE, HOLD MODE, and XY_MOD MODE.  The lower nybble (4 bits) of CMD_HI
specifies which addressing mode is to be used for the Source and Destination
Address pointers.  THIS DETERMINES THE FORMAT IN WHICH THE COUNT, SOURCE AND
DESTINATION ADDRESS, AND MODULO BYTES MUST BE GIVEN.  For LINEAR and HOLD
modes, ABSOLUTE addresses, a 16 bit count, and a ZERO modulo are required.
For MODULO mode, ABSOLUTE addresses, an 8-bit rows-count, an 8 bit columns-
count, and a 16 bit modulo are required.  Finally, if XY_MOD mode is selected,
RELATIVE X-Y coordinate addresses, an 8 bit rows-count, an 8 bit columns-count,
an 8 bit modulo, and an 8 bit XY_MISC must be given.


	ADDRESSING MODES:	HOLD  MOD   ADDRESSING MODE
                                ------------------------------------------
				 0     0    LINEAR  -  CONSECUTIVE
				 0     1    MODULO  -  RECTANGULAR
				 1     0    HOLD    -  CONSTANT
				 1     1    XY_MOD  -  BITMAP RECTANGULAR


There is one more bit in the Source and Destination Address Bank bytes, called
I/O, that tells DMAgic whether to access system memory, or memory mapped I/O
space.  This could be used in conjuntion with the HOLD bit to tranfer to or
from an I/O port, such as an expansion cartridge, hard disk controller, or
network interface.
  
MEMORY LAYOUT

Most routine data transfer operations can be performed with a linear DMAT,
where all addresses are consecutive.  The tansfer length in this case is
specified as a 16 bit count (CNT_LO,CNT_HI), up to a maximum of 64K bytes long
(a value of zero is used to tranfer a full 64K bytes).
With the use of the MODULO register, rectangular blocks of data, such as
characters on a text screen, or graphic images on a bitmap screen, can be
moved as a windowed operation, without disturbing adjacent data.  In this
mode, the length of transfer is specified by COLUMNS and ROWS, with a maximum
of 256 columns(bytes) by 256 rows(lines).

Bitplane display memory in the C64DX is organized as 8 x 8 bit grouped cells
as described in the graphics chapter.  Due to this arrangement, a normal
"linear" transfer would not produce the desired results.  Instead, a special 
XY_MODE has been included to perform the necessary address-munging on the fly.
In this mode, only the X and Y coordinates of the start of the block, as 
well as the screen location, must be specified, and DMAgic will compute the
translated display address.

DMAgic performs all transfers on byte boundaries, and as such, is not a bit
blitter.  However, through careful programming, the coprocessor can perform
many "bit" type operations.  Using the built-in logic operations and bit-
barrel-shifter, individual bits (or pixels) may be manipulated, making
DMAgic particularly well suited to bitplane graphics operations.
