DATA FILES

Disk files are similar to file folders in a file cabinet- both are
organized places to store information.  There are two general types
of data files directly supported by the C64DX computer and DOS: sequential
(SEQ) files and relative (REL) files.  In this section you will be
shown the general file handling commands and then each type of data file
will be discussed.  If you will be using serial bus disk drives, consult
the specific drive's manual for important differences in file handling.

OPENing a File

In order to send data to or receive data from a device other than the
computer's display (the default output device) or its keyboard (the default
input device), you must OPEN a data channel to that device.  Each I/O device
(peripheral) is associated with a unique, preset device number (refer to the
OPEN statement in the BASIC 10.0 Encyclopedia for a list of devices and their
numbers).  A channel is like a pipeline to a particular file on a specific
device, and has another number associated with it, a file number, which you
assign.  In addition, a channel often has certain attributes such as Read,
Write, or record length assigned to it, just as a pipeline might have a valve
controlling direction or quantity of its flow.  You use an open statement to
open a certain type of channel to a particular file on a specified device.
Once a file is open, you need only reference its file number to access it.
The following formats illustrate the various types of disk open statements:

  OPEN file, device, channel, "[[@]drive:]filename,filetype,<mode|reclen>"
  DOPEN#file, "[@]filename,filetype" [,W] [,Lreclen] [,Ddrive] [,Udevice]
  APPEND#file, "filename" [,Ddrive] [,Udevice]

File is an integer from 1 to 255.  This is the logical file number, and it's
used by all subsequent file commands which deal with this file.  Normally the
file number is less than 128, in which case lines printed to the file will be
terminated by the return character, CHR$(13).  If the file number is greater
than 127, then lines will be terminated by the return and linefeed characters,
CHR$(13)+CHR$(10).  The file number can be used by only one file at a time.

The device parameter, sometimes called the primary address, is the number of
the physical device which contains the file.  For disk drives this is normally
an integer from 8 to 30, where 8 is the number of the built-in drive.  In
C64DX mode, a device number of 1 refers to the user's default drive, specified
by the SET DEF command, and it's good practice to use it (although programs
which do use it would not work in C64 mode).  In the case of a dual drive unit,
the drive parameter is used to specify which drive contains the file.  The
default is to access drive 0.  More than one file can be open on a particular
device and drive at any time; how many depends upon the types of files being
opened and the particular disk drive in use.  In any case, the maximum number
of files that can be open simultaneously is 10, which is how many channels the
Kernel operating system can accommodate at one time.

The channel parameter, sometimes called the secondary address, refers to a
particular type of data channel and is normally an integer from 2 to 14.
Channels 0 and 1 are reserved by the DOS as Load and Save channels, and 15
is designated as the DOS command and error channel.  Each channel number can
be used by only one open file at a time; it's common practice to make the
file number and the channel number the same for a data file.  In the case of
DOPEN, you do not have to specify a channel number because the system will
do that for you.

The filename is the name of the file being opened.  The maximum filename
length is 16 characters, exclusive of the pathname, if any.  Pattern matching
may be used to access existing files, but not when creating a new file.
A DOS error will occur if a file opened for Reading does not exist, or if a
file opened for Writing already exists and replacement mode (@) is not
specified.

The filetype is type of file being opened.  By default, the DOS will attempt
to open a Sequential file.  The various filetypes are listed below.  Some of
them will be discussed later in this chapter.

    Filetypes recognized  Filetypes listed        Description of Filetype
          by OPEN               by DIR
    --------------------  ----------------   ---------------------------------
    S, SEQ, SEQUENTIAL		SEQ		sequential data file
    P, PRG, PROGRAM		PRG		program (BASIC or binary) file
    U, USR, USER		USR		user (utility) file
    L, LENGTH			REL		relative data file
    C, CBM			CBM		partition (subdirectory)

Mode is the type of access desired.  By default, the DOS will attempt to
open a file for Reading.  The various modes are listed below.

    R, READ					open a file for reading
    W, WRITE					open a file for writing
    A, APPEND					append data to a file
    M, MODIFY					open a bad file

The Modify mode is used only as a last resort to read data from a file which
has a read error.  If you try this, check the disk status after each byte to
insure the data you read is still valid.

The Append mode is useful for adding data to the end of an existing SEQuential
type file.  The file must already exist, it's opened for Writing, and the file
pointer is placed after the last byte in the file.

In the case of relative files, which are automatically opened for both read
and write access, mode is replaced by the record length, which is required
only when creating a relative file.  When using DOPEN, the record length can
be given explicitly (eg., L100) or given by a variable (eg., L(rl)).  When
using OPEN, the record length must be given as a binary number, usually through
the use of the CHR$() function (eg., "filename,L,"+CHR$(rl)).  Relative files
will be discussed in detail later.

In the case of DOPEN, only Write or Length modes need to be specified
because the system will default to Read mode automatically.  In the case of
APPEND, the system will open an existing SEQuential file in Append mode.

After performing an OPEN, it's important to check both the I/O status (ST)
and disk status (DS) variables for errors.  ST will tell you whether the
disk drive was present or not and if it received the file parameters okay.
DS will tell you if the open command contained any syntax errors, if a
diskette was present, and whether the specified file was found or not.
By using these status variables in conjunction with the TRAP statement it is
possible to write a BASIC 10.0 program that won't crash.  The example below
illustrates how to do this.


   [insert program list A here; use small program font with slashed zeros]
   
Closing a file

It is very important to CLOSE a file when you are finished using it.  Closing
a file frees the channel associated with the file so it can be used again by
the Kernel, flushes the last buffer of data to disk and frees the buffer(s)
allocated to the channel so they can be used again by the DOS, and tells the
DOS to update the system sector on the diskette, if necessary.  Failure to
close a file being written to will create a splat file, which could eventually
corrupt the file and other files on the disk.  The various close statements are:

	CLOSE  file
	DCLOSE #file
	DCLOSE [Udevice]

The file parameter is the file number used when the file was opened.  In the
case of DCLOSE, if you specify a device number then all files open on the
given device will be closed.  If you don't specify a device number, then all
files on the default system drive (see SET DEF) will be closed.

It's good practice to use CLOSE if you used OPEN to open a file, and to use
DCLOSE if you used DOPEN, but the two forms can be used interchangeably.

Some BASIC commands cause the Kernel to free channels without closing files,
and certain disk commands cause the DOS to close files without freeing the
Kernel's channels.  Use commands such as RUN, CLR, LOAD, HEADER, SET DISK,
DCLEAR, and BOOT only after all files have been properly closed.  Also note
that untrapped BASIC errors will leave files open.  Assuming you know which
device was in use, typing  DCLOSE U(device)  in direct mode will close any
files that were open on that device.  There is no harm in closing a file
that is not open.

Writing to a File

The PRINT# command is used to send (transmit) data to a file that was opened
in Write mode previously.  The format for the PRINT# command is:

	PRINT#file, [USING format;] [expression list] [,|;]

This command prints the contents of the expression list to a file, where
file is the logical file number assigned to the file when it was opened.

PRINT# is very much like PRINT, except that the data is written into a file
instead onto the screen.  The data is transmitted in exactly the same format
as it would be printed to the screen display, including all the effects of the
various punctuation characters such as commas and semicolons used to format the
output.  Note that the functions TAB() and SPC() don't necessarily have the
same effect upon files as they would have when printing to the screen.  Refer
to the BASIC 10.0 Encyclopedia for details pertaining to TAB(), SPC(), and
PRINT USING.

If file is less than 128, then lines printed to the file will be terminated by
the return character, CHR$(13).  If the file number is greater than 127, then
lines will be terminated by return and linefeed characters, CHR$(13)+CHR$(10).
If a comma or semicolon follows the last item in the expression list, then no
terminating character(s) will be appended to the output.

The format of the data written to a file deserves important consideration,
since it determines how the data will be read later.  In a simple text file
such as a letter, all you need to be concerned about is the length of each
line.  In a data file containing specific fields which you will want to read
into separate variables later, you must plan in advance how these items will
be separated.  As you will see, just as data is printed the same way to a file
as it to the screen, data is read from a file the same way it's read from the
keyboard.  This means that fields must be separated by return characters or
other punctuation marks, just as you'd expect the user to type the information.
There are also special considerations to be made when writing to RELative
files, which will be discussed later.

It's important to check for errors frequently when writing data to a file.
Errors such as 'disk full' and 'write protected' won't occur until you attempt
to write to the file. Use the sample error program given earlier as an example.


Reading a File

There are two ways to read data from a file: GET# reads the data one byte
(character) at a time, and INPUT# reads a string of characters (field) at a
time.  Either command is used to read (receive) data from a file that was
opened in Read mode previously.  The formats for the commands are:

	       GET#file,   variable list
	[LINE] INPUT#file, variable list

These commands read data from a file into the BASIC variables listed in the
variable list, where file is the logical file number assigned to the file when
it was opened.  These commands can only be used in a program, not in direct
mode.

GET# is very much like GET, and INPUT# is very much like INPUT, except that the
data is read from a file instead of the keyboard.  The data is read from the
file into a variable in exactly the same way as it would be read from the
keyboard, including all the effects of the various punctuation characters such
as commas and return characters used to separate different items.  LINE INPUT#,
which reads all data up to a return or null character, is an alternative to
this.  In any case, the maximum input length is 160 bytes (88 bytes in C64
mode), the size of the Kernel's input buffer.  

Before you can read data from a file, you must know its format.  In a simple
text file such as a letter all you need to be concerned about is the length of
each line.  In a data file containing specific fields which you want to read
into separate variables, you must know how these items are separated.  This
generally means that fields must be separated by return characters or other
punctuation marks.  In addition, the data type must be compatible with the
variable type, which means that only numeric data can be read into a numeric
variable without an error being reported. There are also special considerations
to be made when reading RELative files, which will be discussed later.

GET# works a little differently than GET.  GET# will always return a character
from a file, unlike GET which may return without getting anything.  This means
that unlike GET, where a null character means no character was typed, when GET#
returns a null character it really read a null character, CHR$(0), from the
file.  Therefore, when using GET# to read binary data, a null character
(binary 0) must be converted to CHR$(0) for proper handling by BASIC:

		GET#1,x$: IF x$="" THEN x$=CHR$(0)

It's important to check for errors frequently when reading data from a file. In
particular, when ST has a value of 64 it means the item you just read from the
file was the last item in the file.  If ST has a value of 66, it means there
was no data to read- you have read everything in the file.  There is no other
way to tell when you have reached the end of a file, unless you have written
something to the file to indicate the logical end of the file to your program.
Reading "past" the end of a file won't cause any error, but it can cause your
program to "hang".  The Kernel passes a return character, CHR$(13), if a
channel encounters an I/O error for all subsequent reads from that channel.

If you don't know the format of a data file, you can try to figure it out by
using one of the following techniques.  If the file is a PETSCII text file,
simply TYPE the file and look for patterns in the output:

	TYPE "filename"

Otherwise, use a simple program like this:

	10 DOPEN#1, "filename"			  open the file
	20 DO					  begin loop
	30     GET#1,A$: PRINT A$;		  read a character and print it
	40     LOOP UNTIL ST			  continue until EOF
	50 DCLOSE#1				  close the file

If you TYPE the file or run the program above and get a lot of graphic
characters or unintelligible output, the file probably contains binary data.
The following program will display a binary file's contents in hexadecimal:

	10 DOPEN#1, "filename,P"		  open the file
	20 DO					  begin loop
	30   FOR I=1TO8				     for 8 bytes per line
	40     GET#1,A$				        read a byte
	50     IF ST<>66 THEN PRINT HEX$(ASC(A$))" ";   print HEX value of byte
	60     NEXT					next byte
	70   PRINT					next line
	80   LOOP UNTIL ST 			  continue until EOF
	90 DCLOSE#1	  			  close the file


The RECORD Command

This command is used to access any character in a RELative file.  Relative
files will be discussed in detail later in this chapter.  The format for
the RECORD command is:

	RECORD#file, record [,byte]

File is the logical file number assigned to the file when it was opened. Record
is an integer specifying the number of the record sought, from 1 to 65535.  The
optional byte parameter indicates the character position within the record to
point to, from 1 to 255.  If it's omitted, then the first character in the
specified record is pointed to.  If the given byte pointer is greater than the
record length, a RECORD OVERFLOW error will result.  For either parameter, a
value of zero is the same as a value of one.

When using the resident C64DX DOS, it's possible to get the record length of
an open Relative file using the following code.  Note that you must open the
relative file via OPEN and not DOPEN, because you need to pass the channel
number of the open relative file to the DOS command channel.  When you use
DOPEN, you don't know what the channel number is.

	10 OPEN file, device, channel, "relfile"
	20 DISK "P"+CHR$(channel): GET#file, rl$
	30 PRINT "THE RECORD LENGTH IS";ASC(rl$)
	40 CLOSE file

