File Data Quirks

Novice BASIC programmers are often confused by a several quirks inherent in
the way BASIC handles data read from or written to a file.

Numbers

Unless you specifically write binary data to a file using the CHR$() function,
all data written into a file is in PETSCII string format regardless of the
variable type used in the PRINT# statement.  When the data is read back from
the file, it is converted by BASIC into the appropriate variable type listed
in the INPUT# statement.  The only time this causes a problem is when you
attempt to read data into a numeric type variable and the data contains a
character other than 0-9, plus, minus, space, decimal point, or the letter E
(used for scientific notation).  In this case, BASIC will report a FILE DATA
ERROR and your program will halt.  You must make sure the variable types
used in PRINT# statements when you write to a file match those used in INPUT#
statements when you read that file.

Quotes, Commas, Colons, and Carriage Returns

BASIC interprets certain characters as data delimiters.  A delimiter is a
special character used to separate data fields.  These special characters do
not affect GETs because GET always reads single characters.  But they have a
very great affect upon INPUTs because INPUT reads fields of characters.  A
field is a string of zero, one, or more characters terminated by either a
comma, colon, null, or return character.  Note that a space character does not
terminate a field in the case of INPUT, as it would when READing a DATA
statement.

The only exception to this is when the first character of a field is a quote
character, CHR$(34).  Quotes are used to delimit strings.  A string field
delimited by quotes can contain commas and colons.  When a field beginning
with a quote is read via INPUT#, the leading quote is stripped and all
subsequent characters up to another quote, a null, or return character are
accepted as part of the field.

File Data Solutions

All the aforementioned quirks can be avoided easily.  First, write only one
variable with each PRINT# statement.  The terminating return character is
automatic and explicit.  Second, write and read numeric fields as numbers.
Third, read string fields with the LINE INPUT# statement.  This lets you
read all characters, including quotes, commas, and colons without any
addition effort or obtuse parsing algorithms.  If you code you program in
this very straightforward manner it will probably work the first time, be
faster, and be easier to read, write, and maintain.

If you insist upon writing several different fields with one PRINT# statement,
consider using the PRINT USING statement.  It allows you specify the format
of the output explicitly, it lets you organize your fields clearly, and it lets
you insert delimiters between fields easily.  You will still have to worry
about special characters in string fields, but at least you know exactly
where each field begins and ends:

	100 DOPEN#1, "price list",W
	110 PRINT"Enter ITEM#, COLOUR, and PRICE separated by commas"
	120 PRINT"End input by entering 0,0,0"
	130  DO
	140    INPUT item, colour$, price
	150    IF item=0 THEN EXIT
	160    PRINT#1, USING"#### , ##### , ###.##"; item, colour$, price
	170    LOOP
	180  DCLOSE#1
	190 :

	200 PRINT
	210 TYPE "price list"
	220 :

	300 DOPEN#1, "price list"
	310 PRINT CHR$(2);" ITEM#   COLOUR    PRICE "
	320 DO
	330   INPUT#1, item, colour$, price
	340   PRINT    item, colour$, price
	350   LOOP UNTIL ST
	360 DCLOSE#1
	370 :
	380 END
