OPEN - Open a channel to a device or disk file

	OPEN channel_number, device [,secondary_address [,{filespec|command}]]

Before a program can access a device or a file, an I/O channel must be opened
to it, through which the computer and device can communicate.  When you open a
channel, you assign a channel_number to the channel.  The channel_number is
used by all future I/O commands to access the particular device or file (see
the PRINT#, INPUT#, GET#, and CMD commands).  Use the ST status variable to
check the I/O status of an OPEN command.

A channel_number can be used by one open channel at a time; if another OPEN
statement uses a channel_number that is already in use, a 'FILE OPEN' error
results.  The system allows up to 10 channels to be open simultaneously.  If
you attempt to open more than 10, a 'TOO MANY FILES' error results.  When you
are done using a channel, use the CLOSE command to close the channel and free
the channel_number.  The channel_number is an integer from 1 to 255 (0 is
reserved by the system).  Normally, channel numbers are less than 128; channels
numbers greater than 127 are only used translate carriage return characters
into return/linefeed characters.

Different devices require different forms of the OPEN command.  The device
number assignments in the C64DX are:

device	 description			       typical OPEN syntax	mode
------	 -----------------------------------   -------------------	----
 0	 Keyboard			       OPEN n,0			input
 1	 Default system drive, usually #8      OPEN n,1,sa,"file"	(I/O)
 2	 RS232				       OPEN n,2,n,"config"	(I/O)
 3	 Text screen			       OPEN n,3			(I/O)
 4 - 7	 Serial bus (usually printers)	       OPEN n,4,sa,"data"	(I/O)
 5 - 31	 Serial bus (usually disk drives)      OPEN n,8,sa,"file"	(I/O)

Opening channels to the keyboard or text screen

Normally, channels are not opened to the keyboard or the screen, since these
devices are the default input and output devices used by the INPUT, GET, and
PRINT statements.  It's not necessary to communicate to these devices through
channels.  

Opening channels to printers:

The OPEN syntax for a typical Commodore serial bus printer is:

		OPEN channel, device [,sa [,"params"] ]

where sa selects a particular printer mode (for example, 0=uppercase/graphic
mode, 7=upper/lowercase mode) and params is usually a list of CHR$() values
(similar to escape sequences) used to configure the printer.  Refer to your
printer manual for details.

Opening channels to disk drives:

The C64DX built-in drive defaults to device 8 and the optional, external 1565
drive defaults to device 9.  Use the SET DISK command to change their device
numbers.  Use the SET DEF command to specify a different default system drive.
The default system drive is referenced by commands such as DIR, DELETE, TYPE,
DOPEN, etc., where the device number parameter is optional.

The OPEN syntax for a typical disk file operation is:

		OPEN  file, device, sa, "[drive:] filename [,type] [,mode]"

Refer to your DOS manual for details regarding disk files or command channels,
or see Chapter 7, Using the DOS.  See also the DOPEN and APPEND commands.

Opening an RS232 (modem) channel:

The C64DX has a built-in UART, a hardware serial communications device.  You
can communicate with most personal computers with nothing more than a cable
connecting the two systems, but to communicate with other terminals, modems,
or mainframe computers you will need an optional adapter to convert the C64DX's
TTL signals into true RS232 level signals.  The OPEN command has the following
syntax:

		OPEN channel, 2, sa, [configuration_list]

where sa is a number 0-255 (it's unused but required) and the config_list is a
list of bytes {CHR$()+CHR$()...} used to setup the channel:

	byte#	description		values
	-----	----------------------	----------------------------------
	 1	baud rate		(0-16, where 16=MIDI rate)
	 2	word length		(0=8, 1=7, 2=6, 3=5 bit words)
	 3	parity			(0=none, 2=even, 3=odd)
	 4	stop bits 		(not used, based upon word length)
	 5	duplex			(0=half, 1-full)
	 6	xline			(0=3-line, 1=x-line)
	 7	xon char		(0=incoming flow control disabled)
	 8	xoff char		(0=outgoing flow control disabled)
	 9,10	input  buffer pointer	(page lo, hi)
	11,12	output buffer pointer	(page lo, hi)
	13	high water mark		(point at which xoff is xmitted)
	14	low  water mark		(point at which xon  is xmitted)

The default RS232 setup is 1200 baud, 8-bit words, 1-stop bit, no parity,
XON/XOFF.  The OS automatically adjusts for 50/60Hz environments.

	OPEN n, 2, 2, CHR$(14)		open a 9600 baud, 8N1 channel

Refer to the UART register description in Appendix K for more details.
